==> v1.1.6.tar.gz: twitch-cli-1.1.6/internal/mock_api/authentication/authentication.go <==
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
package authentication

import (
	"context"
	"errors"
	"fmt"
	"log"
	"net/http"
	"strings"
	"time"

	"github.com/twitchdev/twitch-cli/internal/database"
	"github.com/twitchdev/twitch-cli/internal/mock_api"
	"github.com/twitchdev/twitch-cli/internal/mock_api/mock_errors"
)

type UserAuthentication struct {
	Scopes   []string
	UserID   string
	ClientID string
}

func AuthenticationMiddleware(next mock_api.MockEndpoint) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		db := r.Context().Value("db").(database.CLIDatabase)

		// skip auth check for unsupported methods
		if next.ValidMethod(r.Method) == false {
			w.WriteHeader(http.StatusMethodNotAllowed)
			return
		}

		clientID := r.Header.Get("Client-ID")
		bearerToken := r.Header.Get("Authorization")
		unauthroizedError := mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Missing Client ID or OAuth token")
		if clientID == "" || bearerToken == "" || len(bearerToken) < 7 {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(unauthroizedError)
			return
		}

		prefix := strings.ToLower(bearerToken[:6])
		token := bearerToken[7:]

		// check if the client ID is invalid or missing the proper token prefix
		if len(clientID) < 30 || prefix != "bearer" {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(unauthroizedError)
			return
		}

		// get the authorization from the db
		auth, err := db.NewQuery(r, 100).GetAuthorizationByToken(token)
		if err != nil {
			log.Print(err)
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		// check if invalid
		if auth.Token == "" {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Invalid OAuth token"))
			return
		}

		// check for mismatches
		if auth.ClientID != clientID {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Missing Client ID or OAuth token"))
			return
		}

		// check if expired
		expiration, err := time.Parse(time.RFC3339, auth.ExpiresAt)
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		if time.Now().After(expiration) {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Token expired"))
			return
		}

		// pass as context
		authContext := UserAuthentication{
			Scopes:   strings.Split(auth.Scopes, " "),
			UserID:   auth.UserID,
			ClientID: auth.ClientID,
		}

		if authContext.HasOneOfRequiredScope(next.GetRequiredScopes(r.Method)) == false {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), fmt.Sprintf("Missing required scope %v", strings.Join(next.GetRequiredScopes(r.Method), " or "))))
			return
		}

		r = r.WithContext(context.WithValue(r.Context(), "trace-id", "1234"))
		r = r.WithContext(context.WithValue(r.Context(), "auth", authContext))

		next.ServeHTTP(w, r)
	})
}

func (u UserAuthentication) HasScope(scope string) bool {
	for _, s := range u.Scopes {
		if s == scope {
			return true
		}
	}
	return false
}

func (u UserAuthentication) HasOneOfRequiredScope(scopes []string) bool {
	if len(scopes) == 0 {
		return true
	}

	for _, s := range scopes {
		for _, us := range u.Scopes {
			if s == us {
				return true
			}
		}
	}
	return false
}

func (u *UserAuthentication) MatchesBroadcasterIDParam(r *http.Request) bool {
	bid := r.URL.Query().Get("broadcaster_id")
	if bid == "" || bid != u.UserID {
		return false
	}
	return true
}

==> v1.1.24.tar.gz: twitch-cli-1.1.24/internal/mock_api/authentication/authentication.go <==
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
package authentication

import (
	"context"
	"errors"
	"fmt"
	"log"
	"net/http"
	"strings"
	"time"

	"github.com/twitchdev/twitch-cli/internal/database"
	"github.com/twitchdev/twitch-cli/internal/mock_api"
	"github.com/twitchdev/twitch-cli/internal/mock_api/mock_errors"
)

type UserAuthentication struct {
	Scopes   []string
	UserID   string
	ClientID string
}

func AuthenticationMiddleware(next mock_api.MockEndpoint) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		db := r.Context().Value("db").(database.CLIDatabase)

		// skip auth check for unsupported methods
		if next.ValidMethod(r.Method) == false {
			w.WriteHeader(http.StatusMethodNotAllowed)
			return
		}

		clientID := r.Header.Get("Client-ID")
		bearerToken := r.Header.Get("Authorization")
		unauthroizedError := mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Missing Client ID or OAuth token")
		if clientID == "" || bearerToken == "" || len(bearerToken) < 7 {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(unauthroizedError)
			return
		}

		prefix := strings.ToLower(bearerToken[:6])
		token := bearerToken[7:]

		// check if the client ID is invalid or missing the proper token prefix
		if len(clientID) < 30 || prefix != "bearer" {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(unauthroizedError)
			return
		}

		// get the authorization from the db
		auth, err := db.NewQuery(r, 100).GetAuthorizationByToken(token)
		if err != nil {
			log.Print(err)
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		// check if invalid
		if auth.Token == "" {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Invalid OAuth token"))
			return
		}

		// check for mismatches
		if auth.ClientID != clientID {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Missing Client ID or OAuth token"))
			return
		}

		// check if expired
		expiration, err := time.Parse(time.RFC3339, auth.ExpiresAt)
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		if time.Now().After(expiration) {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), "Token expired"))
			return
		}

		// pass as context
		authContext := UserAuthentication{
			Scopes:   strings.Split(auth.Scopes, " "),
			UserID:   auth.UserID,
			ClientID: auth.ClientID,
		}

		if authContext.HasOneOfRequiredScope(next.GetRequiredScopes(r.Method)) == false {
			w.WriteHeader(http.StatusUnauthorized)
			w.Write(mock_errors.GetErrorBytes(http.StatusUnauthorized, errors.New("Unauthorized"), fmt.Sprintf("Missing required scope %v", strings.Join(next.GetRequiredScopes(r.Method), " or "))))
			return
		}

		r = r.WithContext(context.WithValue(r.Context(), "trace-id", "1234"))
		r = r.WithContext(context.WithValue(r.Context(), "auth", authContext))

		next.ServeHTTP(w, r)
	})
}

func (u UserAuthentication) HasScope(scope string) bool {
	for _, s := range u.Scopes {
		if s == scope {
			return true
		}
	}
	return false
}

func (u UserAuthentication) HasOneOfRequiredScope(scopes []string) bool {
	if len(scopes) == 0 {
		return true
	}

	for _, s := range scopes {
		for _, us := range u.Scopes {
			if s == us {
				return true
			}
		}
	}
	return false
}

func (u *UserAuthentication) MatchesBroadcasterIDParam(r *http.Request) bool {
	return u.MatchesSpecifiedIDParam(r, "broadcaster_id")
}

func (u *UserAuthentication) MatchesModeratorIDParam(r *http.Request) bool {
	return u.MatchesSpecifiedIDParam(r, "moderator_id")
}

func (u *UserAuthentication) MatchesUserIDParam(r *http.Request) bool {
	return u.MatchesSpecifiedIDParam(r, "user_id")
}

func (u *UserAuthentication) MatchesSpecifiedIDParam(r *http.Request, param string) bool {
	id := r.URL.Query().Get(param)
	if id == "" || id != u.UserID {
		return false
	}
	return true
}
